
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include <ctype.h>

#include "xstd/h/strstream.h"
#include "xstd/Checksum.h"
#include "xstd/StrIdentifier.h"
#include "xstd/gadgets.h"
#include "base/BStream.h"
#include "runtime/IOBuf.h"

#include "csm/XmlTagParser.h"
#include "csm/InjectIter.h"
#include "csm/cdbEntries.h"


/* CdbeBlob */

void CdbeBlob::image(const String &anImage) {
	theImage = anImage;
}

OBStream &CdbeBlob::store(OBStream &ol) const {
	return ol << theImage;
}

IBStream &CdbeBlob::load(IBStream &il) {
	return il >> theImage;
}

ostream &CdbeBlob::print(ostream &os) const {
	return os << theImage;
}

void CdbeBlob::pour(CdbEntryPrnOpt &opt) const {
	Assert(opt.entryOff < theImage.len());
	const Size len = Min(Size(theImage.len())-opt.entryOff, opt.sizeMax, opt.buf->spaceSize());

	if (theImage && len)
		opt.buf->append(theImage.data() + opt.entryOff, len);
}


/* CdbeText */

void CdbeText::pour(CdbEntryPrnOpt &opt) const {
	const Size mark = opt.buf->contSize();
	CdbeBlob::pour(opt);
	if (opt.injector)
		opt.injector->inject(*opt.buf, 0, opt.buf->contSize() - mark);
}


/* CdbeTag */
  
StrIdentifier *CdbeTag::TheReplIdentifier = 0;
Array<String*> CdbeTag::TheReplacements;

void CdbeTag::image(const String &anImage) {
	const char *tagB = anImage.cstr();
	const char *tagE = anImage.cstr() + anImage.len();

	static XmlTagParser parser;
	if (!parser.parse(tagB+1, tagE-1)) {
		CdbeBlob::image(anImage);
		return;
	}

	if (!isalpha(*parser.tagname().nameBeg)) { // closing tag, comment, etc.
		CdbeBlob::image(anImage);
		return;
	}

	String keyPfx;
	keyPfx.append(parser.tagname().nameBeg, parser.tagname().nameLen);
	keyPfx += '.';

	// replace src="url" with src="/pg/embed/..."
	String newImage;
	const char *lastCopy = tagB;
	for (int i = 0; i < parser.attrCount(); ++i) {
		XmlTagParser::Token attr = parser.attr(i);
		if (attr.valBeg && *attr.valBeg != '#') { // skip URLs pointing to self
			const String key = keyPfx + String(attr.nameBeg, attr.nameLen);
			if (const String *replacement = AttrValReplacement(key)) {
				newImage += anImage(lastCopy - tagB, attr.valBeg - tagB);
				newImage += *replacement;
				newImage += ',';
				newImage += calcCheckSum(attr.valBeg, attr.valLen);
				lastCopy = attr.valBeg + attr.valLen;
			}
		}
	}
	newImage += anImage(lastCopy - tagB, tagE - tagB);
	CdbeBlob::image(newImage);
}

String CdbeTag::calcCheckSum(const char *str, int len) const {
	char buf[64];
	ostrstream os(buf, sizeof(buf));

	ChecksumAlg a;
	a.update(str, len);
	a.final();
	PrintBase64(os, a.sum().image(), a.sum().size());
	os << ends;
	
	return String(buf, strlen(buf));
}

int CdbeTag::RegReplacement(const String &key, const String &ctype) {
	Assert(TheReplIdentifier);
	const int id = TheReplIdentifier->add(key);
	TheReplacements.put(new String(ctype), id);
	return id;
}

const String *CdbeTag::AttrValReplacement(const String &key) {
	if (!TheReplIdentifier) {
		TheReplIdentifier = new StrIdentifier;
		const String embedHtml = "/pg/embed/html";
		const String embedImage = "/pg/embed/image";
		const String embedData = "/pg/embed/data";
		RegReplacement("applet.archive", embedData);
		RegReplacement("body.background", embedImage);
		RegReplacement("frame.src", embedHtml);
		RegReplacement("iframe.src", embedHtml);
		RegReplacement("img.src", embedImage);
		RegReplacement("img.usemap", embedHtml);
		RegReplacement("input.src", embedImage);
		RegReplacement("input.usemap", embedHtml);
		RegReplacement("layer.src", embedHtml);
		RegReplacement("object.data", embedData);
		RegReplacement("script.src", embedHtml);
		TheReplIdentifier->optimize();
	}

	const int idx = TheReplIdentifier->lookup(key);
	if (idx > 0) {
		Assert(idx < TheReplacements.count());
		return TheReplacements[idx];
	}
	return 0;
}

