
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include <fstream.h>

#include "xml/XmlAttr.h"
#include "loganalyzers/LoadStex.h"
#include "loganalyzers/ReportBlob.h"
#include "loganalyzers/BlobDb.h"
#include "loganalyzers/RepOpts.h"
#include "loganalyzers/PhaseInfo.h"
#include "loganalyzers/PhaseTrace.h"
#include "loganalyzers/LoadTraceFig.h"


LoadTraceFig::LoadTraceFig(): theStex1(0), theStex2(0), thePhase(0), theTrace(0) {
}

void LoadTraceFig::stats(const Stex *aStex1, const Stex *aStex2, const PhaseInfo *aPhase) {
	theStex1 = aStex1;
	theStex2 = aStex2;
	thePhase = aPhase;
	theTrace = &thePhase->trace();
	Assert(theTrace);
}

void LoadTraceFig::setCtrlOptions() {
	ReportTraceFigure::setCtrlOptions();
	*theCtrlFile << "set size 1.0,0.5" << endl;
	*theCtrlFile << "set xlabel 'minutes'" << endl;
	*theCtrlFile << "set ylabel 'rate, #/sec'" << endl;
	*theCtrlFile << "set y2label 'bandwidth, Mbits/sec'" << endl;
	*theCtrlFile << "set ytics nomirror" << endl;
	*theCtrlFile << "set y2tics nomirror" << endl;
}

int LoadTraceFig::createCtrlFile() {
	if (ReportTraceFigure::createCtrlFile() < 0)
		return -1;


	*theCtrlFile << "plot '-' using 1:2 axis x1y1 title '" << theStex1->name() 
		<< " rate (left y-axis)' with lines lt 3, \\" << endl;
	*theCtrlFile << "\t '-' using 1:3 axis x1y2 title '" << theStex1->name() 
		<< " bwidth (right y-axis)' with lines lt 4, \\" << endl;

	*theCtrlFile << "\t '-' using 1:2 axis x1y1 title '" << theStex2->name()
		<< " rate' with lines lt 1, \\" << endl;
	*theCtrlFile << "\t '-' using 1:3 axis x1y2 title '" << theStex2->name()
		<< " bwidth' with lines lt 2" << endl;

	int pointCount = 0;
	pointCount += dumpDataLines(theStex1);
	*theCtrlFile << 'e' << endl;
	dumpDataLines(theStex1);

	*theCtrlFile << 'e' << endl;

	pointCount += dumpDataLines(theStex2);
	*theCtrlFile << 'e' << endl;
	dumpDataLines(theStex2);

	return pointCount;
}

int LoadTraceFig::dumpDataLines(const LoadStex *stex) {
	int pointCount = 0;
	for (int i = 0; i < theTrace->count(); ++i)
		pointCount += dumpDataLine(stex, theTrace->winPos(i), theTrace->winStats(i));

	return pointCount;
}

int LoadTraceFig::dumpDataLine(const LoadStex *stex, Time stamp, const StatIntvlRec &r) {
	if (true) {
		const double rate = stex->rate(r);
		const double bwidth = stex->bwidth(r);
		dumpTime(stamp);
		*theCtrlFile << ' ' << rate << ' ' << (bwidth*8/1024/1024) << endl;
		return 1;
	}
	return 0;	
}
