
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "pgl/ContentSym.h"
#include "pgl/PglIntSym.h"
#include "pgl/PglTimeSym.h"
#include "pgl/PglRec.h"
#include "pgl/PglArraySym.h"
#include "pgl/PglStringSym.h"
#include "pgl/ServerSym.h"



const String ServerSym::TheType = "Server";

static String strContentArr = "Content[]";
static String strAccept_lmt = "accept_lmt";
static String strContents = "contents";
static String strDirect_access = "direct_access";
static String strRep_types = "rep_types";
static String strStringArr = "string[]";


ServerSym::ServerSym(): AgentSym(TheType) {
	theRec->bAdd(IntSym::TheType, strAccept_lmt, 0);
	theRec->bAdd(strContentArr, strContents, 0);
	theRec->bAdd(strContentArr, strDirect_access, 0);
	theRec->bAdd(strStringArr, strRep_types, 0);
}

ServerSym::ServerSym(const String aType, PglRec *aRec):	AgentSym(aType, aRec) {
}

bool ServerSym::isA(const String &type) const {
	return AgentSym::isA(type) || type == TheType;
}

SynSym *ServerSym::dupe(const String &type) const {
	if (isA(type))
		return new ServerSym(this->type(), theRec->clone());
	return AgentSym::dupe(type);
}

bool ServerSym::acceptLmt(int &lmt) const {
	return getInt(strAccept_lmt, lmt);
}

bool ServerSym::contents(Array<ContentSym*> &ccfgs, Array<double> &cprobs) const {
	SynSymTblItem *conti = 0;
	Assert(theRec->find(strContents, conti));
	if (!conti->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)conti->sym()->cast(ArraySym::TheType);
	a.copyProbs(cprobs);
	ArraySymExportM(ContentSym, a, ContentSym::TheType, ccfgs);
	return true;
}

bool ServerSym::directAccess(Array<ContentSym*> &dacfgs, Array<double> &daprobs) const {
	SynSymTblItem *dai = 0;
	Assert(theRec->find(strDirect_access, dai));

	const bool hasDa = dai->sym() != 0;

	// use contents array if direct_access is not defined
	if (!hasDa)
		Assert(theRec->find(strContents, dai));
	if (!dai->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)dai->sym()->cast(ArraySym::TheType);
	ArraySymExportM(ContentSym, a, ContentSym::TheType, dacfgs);

	daprobs.reset();
	if (hasDa && a.probsSet())
		a.copyProbs(daprobs);

	return true;
}

String ServerSym::msgTypesField() const {
	return strRep_types;
}
